# sim800.py

import RPi.GPIO as GPIO
from time import sleep
import serial

P_POWER = 11 # Power pin
P_RESET = 12 # Reset pin

class SIM800:
    def __init__(self, verbose = False):
        self.verbose = verbose
        self.debug("Finding serial port.")
        serialPort = "/dev/ttyS0"
        try:
            self.debug("Trying with port " + serialPort)
            self.ser = serial.Serial(serialPort, baudrate = 9600, timeout = 5)
        except: # Wrong serial port
            serialPort = "/dev/ttyAMA0"
            self.debug("Failed. Trying with port " + serialPort)
            try:
                self.ser = serial.Serial(serialPort, baudrate = 9600, timeout = 5)
            except: # Wrong serial port
                print "Fatal error. Serial port not found."
                self.ser = None
                return
        self.debug("Serial port " + serialPort + " found.")
            
        GPIO.setwarnings(False)
        GPIO.setmode(GPIO.BOARD)
        GPIO.setup(P_RESET, GPIO.OUT)
        GPIO.setup(P_POWER, GPIO.OUT)

    def debug(self, text):
        if self.verbose:
            print "Debug:---", text

    def checkReady(self):
        cmd = "AT\r"
        self.debug("Calling checkReady(). Cmd: " + cmd)
        self.ser.write(cmd)
        sleep(2)
        reply = self.ser.read(self.ser.inWaiting())
        self.debug("reply:\n" + reply)
        return ("OK" in reply)

    def reset(self):
        self.debug("Calling reset()")
        maxRetries = 2
        retry = 0
        while retry < maxRetries:
            rc = self._reset()
            if rc:
                return True
            retry += 1
            self.debug("Retry #" + str(retry))
        return False   
            
    def _reset(self):
        self.togglePower() # we don't know if turn off or on
        self.ser.flushInput()
        if self.checkReady():  # Turned on now
            self.debug("Is turned on")
            sleep(2)
            return True
        self.debug("Must turn on")
        self.togglePower() # turn on
        sleep(2)
        return self.checkReady()    

    def turnOff(self):
        self.debug("Calling turnOff()")
        if self.checkReady():  # Turned on
            self.debug("Modem on state detected")
            self.togglePower()
        sleep(2)

    def checkRegistration(self):
        self.debug("Calling checkRegistration()")
        maxRetries = 5
        retry = 0
        while retry < maxRetries:
            cmd = 'AT+CREG?\r'
            self.debug("Check registration. Cmd: " + cmd)
            self.ser.write(cmd)
            sleep(2)
            reply = self.ser.read(self.ser.inWaiting())
            self.debug("reply:\n" + reply)
            if ("0,1" in reply):
                return True
            retry += 1
            self.debug("Retry #: " + str(retry))

    def resetHardware(self):
        self.debug("Calling resetHardware()")
        GPIO.output(P_RESET, GPIO.LOW)
        sleep(1)
        GPIO.output(P_RESET, GPIO.HIGH)
        sleep(5)
        GPIO.output(P_RESET, GPIO.LOW)
        sleep(1)
    
    def togglePower(self):
        self.debug("Calling togglePower()")
        GPIO.output(P_POWER, GPIO.LOW)
        sleep(1)
        GPIO.output(P_POWER, GPIO.HIGH)
        sleep(5)
        GPIO.output(P_POWER, GPIO.LOW)
        sleep(1)
     
    def login(self, apn):
        cmd = 'AT+CSTT="' + apn + '"\r'
        self.debug("Calling login with apn: " + apn + " (no userid/password needed). Cmd: " + cmd)
        self.ser.write(cmd)
        sleep(2)
        reply = self.ser.read(self.ser.inWaiting())
        self.debug("reply:\n" + reply)
        sleep(5)
        return ("OK" in reply)
    
    def startNetwork(self):        
        cmd = "AT+CIICR\r"
        self.debug("Calling startNetwork(). Cmd: " + cmd)
        self.ser.write(cmd)
        sleep(2)
        reply = self.ser.read(self.ser.inWaiting())
        self.debug("reply:\n" + reply)
        sleep(5)
        return ("OK" in reply)
        
    def connectTCP(self, host, port):
        cmd = 'AT+CIPSTART="TCP","' + host + '","' + str(port) + '"\r'
        self.debug("Calling connectTCP with host: " + host + ", port: " + str(port) + ". CMD: " + cmd)
        self.ser.write(cmd)
        sleep(5)
        reply = self.ser.read(self.ser.inWaiting())
        self.debug("reply:\n" + reply)
        return reply
    
    def sendHTTPRequest(self, host, request):
        cmd = "AT+CIPSEND\r"
        self.debug("Calling sendHTTPRequest() (preparing). Cmd: " + cmd)
        self.ser.write(cmd)
        sleep(4)
        request = "GET " + request + " HTTP/1.1\r\nHost: " + host + "\r\n\r\n" 
        self.debug("Request: " + request)
        self.ser.write(request + chr(26))  # data<^Z>
        sleep(2)
        
    def closeTCP(self, showResponse = False):
        cmd = "AT+CIPCLOSE=1\r"
        self.debug("Calling closeTCP(). Cmd: " + cmd)
        self.ser.write(cmd) 
        sleep(2)
        reply = self.ser.read(self.ser.inWaiting())
        self.debug("reply:\n" + reply)
        if showResponse:
            print "Server reponse:\n" + reply[(reply.index("SEND OK") + 9):]
        sleep(2)
        
    def getIPStatus(self):
        cmd = "AT+CIPSTATUS\n"
        self.debug("Calling getIPStatus(). Cmd: " + cmd)
        self.ser.write(cmd)
        sleep(2)
        reply = self.ser.read(self.ser.inWaiting())
        self.debug("Returning all messages:\n" + reply)
        return reply

    def getIPAddress(self):        
        cmd = "AT+CIFSR\r"
        self.debug("Calling getIPAddress(). Cmd: " + cmd)
        self.ser.write(cmd)
        sleep(2)
        reply = self.ser.read(self.ser.inWaiting())
        self.debug("reply:\n" + reply)
        return reply[11:-2]
